@@ -20,7 +20,7 @@
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
 
-static char rcsid[] ="$Id: tree.c,v 1.5 1997/03/12 03:00:56 eric Rel $";
+static char rcsid[] ="$Id: tree.c,v 1.9 1997/06/27 11:24:20 eric Exp $";
 
 /* ADD_FILES changes made by Ross Biro biro@yggdrasil.com 2/23/95 */
 
@@ -142,6 +142,9 @@ void FDECL1(sort_n_finish, struct directory *, this_dir)
   time_t		    current_time;
   struct directory_entry  * table;
   int			    count;
+  int			    d1;
+  int			    d2;
+  int			    d3;
   int			    new_reclen;
   char			 *  c;
   int			    tablesize = 0;
@@ -193,36 +196,57 @@ void FDECL1(sort_n_finish, struct directory *, this_dir)
 	{
 	  if(strlen(rootname) > 27) rootname[27] = 0;
 	}
+
+      /*
+       * Strip off the non-significant part of the name so that we are left
+       * with a sensible root filename.  If we don't find a '.', then try
+       * a ';'.
+       */
       c  = strchr(rootname, '.');
-      if (c) *c = 0;
-      count = 0;
-      while(count < 0x1000)
+      if (c) 
+	*c = 0;
+      else
 	{
-	  sprintf(newname,"%s.%3.3X%s", rootname,  count,
-		  (s_entry->isorec.flags[0] == 2 || 
-		   omit_version_number ? "" : ";1"));
-	  
+	  c  = strchr(rootname, ';');
+	  if (c) *c = 0;
+	}
+      for(d1 = 0; d1 < 36; d1++)
+	{
+	  for(d2 = 0; d2 < 36; d2++)
+	    {
+	      for(d3 = 0; d3 < 36; d3++)
+		{
+		  sprintf(newname,"%s.%c%c%c%s", rootname,  
+			  (d1 <= 9 ? '0' + d1 : 'A' + d1 - 10),
+			  (d2 <= 9 ? '0' + d2 : 'A' + d2 - 10),
+			  (d3 <= 9 ? '0' + d3 : 'A' + d3 - 10),
+			  (s_entry->isorec.flags[0] == 2 || 
+			   omit_version_number ? "" : ";1"));
+		  
 #ifdef VMS
-	  /* Sigh.  VAXCRTL seems to be broken here */
-	  {
-	    int ijk = 0;
-	    while(newname[ijk]) 
-	      {
-		if(newname[ijk] == ' ') newname[ijk] = '0';
-		ijk++;
-	      }
-	  }
+		  /* Sigh.  VAXCRTL seems to be broken here */
+		  {
+		    int ijk = 0;
+		    while(newname[ijk]) 
+		      {
+			if(newname[ijk] == ' ') newname[ijk] = '0';
+			ijk++;
+		      }
+		  }
 #endif
-	  
-	  if(!find_file_hash(newname)) break;
-	  count++;
+		  
+		  if(!find_file_hash(newname)) goto got_valid_name;
+		}
+	    }
 	}
-      if(count >= 0x1000)
-	{
-	  fprintf(stderr,"Unable to  generate unique  name for file %s\n", s_entry->name);
-	  exit(1);
-	}
-      
+
+      /*
+       * If we fell off the bottom here, we were in real trouble.
+       */
+      fprintf(stderr,"Unable to  generate unique  name for file %s\n", s_entry->name);
+      exit(1);
+
+got_valid_name:      
       /* 
        * OK, now we have a good replacement name.  Now decide which one
        * of these two beasts should get the name changed 
@@ -293,7 +317,7 @@ void FDECL1(sort_n_finish, struct directory *, this_dir)
       table->filedir = root;
       table->isorec.flags[0] = 0;
       table->priority  = 32768;
-      iso9660_date(table->isorec.date, fstatbuf.st_ctime);
+      iso9660_date(table->isorec.date, fstatbuf.st_mtime);
       table->inode = TABLE_INODE;
       table->dev = (dev_t) UNCACHED_DEVICE;
       set_723(table->isorec.volume_sequence_number, DEF_VSN);
@@ -303,7 +327,7 @@ void FDECL1(sort_n_finish, struct directory *, this_dir)
       table->name = strdup("<translation table>");
       table->table = (char *) e_malloc(ROUND_UP(tablesize));
       memset(table->table, 0, ROUND_UP(tablesize));
-      iso9660_file_length  ("TRANS.TBL", table, 1);
+      iso9660_file_length  ("TRANS.TBL", table, 0);
       
       if(use_RockRidge)
 	{
@@ -928,7 +952,7 @@ FDECL3(scan_directory_tree,char *, path, struct directory_entry *, de,
     s_entry->filedir = this_dir;
     s_entry->isorec.flags[0] = 0;
     s_entry->isorec.ext_attr_length[0] = 0;
-    iso9660_date(s_entry->isorec.date, statbuf.st_ctime);
+    iso9660_date(s_entry->isorec.date, statbuf.st_mtime);
     s_entry->isorec.file_unit_size[0] = 0;
     s_entry->isorec.interleave[0] = 0;
     if(parent && parent ==  reloc_dir && strcmp(d_entry->d_name,  "..") == 0){
